(ns rlm.clojure-example)

(defn curry
  "takes a function with a fixed number of args and modifies
   it to that it will be automaticaly curried when called with
   less args"
  ([num-args f & eval-list]
     (if (= num-args (count eval-list))
       (apply f eval-list)
       (fn [& args]
	 (apply curry num-args f (concat eval-list args))))))

(defn mix 
  "Takes any number of mathematically equal functions with
   possibly different run-times and returns a function that
   runs each in a separate thread, returns the result from
   the first thread which finishes, and cancels the other threads."
  {:author "Robert McIntyre"}
  ([& functions]
     (fn [& args]
       (let [result (promise)
	     futures
	     (doall (for [fun functions]
		      (future (deliver result (apply fun args)))))
	     answer @result]
	 (dorun (map future-cancel futures))
	 answer))))

(defn memoize*
  "Returns a memoized version of a referentially transparent
  function. The memoized version of the function keeps a cache of the
  mapping from argument to results and, when calls with the same
  arguments are repeated often, has higher performance at the expense
  of higher memory use."
  {:added "1.0"}
  [f]
  (let [mem (atom {})]
    (fn [& args]
      (if-let [e (find @mem args)]
        (val e)
        (let [ret (apply f args)]
          (swap! mem assoc args ret)
          ret)))))

;;immutable data structures
(def r {:a :a :b :b})
(def b (assoc r :c :c))

(defn collapse [coll & head]
  (if (= (count coll) 0)
    (first head)
    (let [x (first coll)]
      (if (list? x)
	(collapse (concat x (rest coll)) (first head))
	(collapse (rest coll) (conj (vec (first head)) x))))))
;;email clojure group about this


(defn collapse* [coll]
  (loop [head [] body coll]
    (let [x (first body)]
      (cond (= (count body) 0) head
	    (list? x) (recur head (concat x (next body)))
	    true      (recur (conj head x) (next body))))))
	

  


