(ns rlm.simple-drawing)



(def max-x 500)
(def max-y 500)
(def shapes (atom []))


(defn render-shape
  [graphics [shape-name x y w h r g b a]]
  (.setColor graphics (java.awt.Color. r g b a))
  (case shape-name
	rect (.fillRect graphics x y w h)
	oval (.fillOval graphics x y w h)
	line (.drawLine graphics x y w h)))

(def panel
     (let [jp (proxy [javax.swing.JPanel]
		  []
		(getPreferredSize [] (java.awt.Dimension. max-x max-y))
		(paint [g]
		       (render-shape g ['rect 0 0 max-x max-y 255 255 255 255])
		       (doall (map #(render-shape g %) @shapes))))]
       (doto (new javax.swing.JFrame "My graphics window")
	 (.setSize max-x max-y)
	 (.add jp)
	 (.setVisible true))
       jp))


(defn draw-shape [& shape]
  "Adds a shape to the current drawing. The first argument should be one of
the following symbols: rect, oval, or line. For rect or oval  this should
be followed by: x, y, width, height, red, green, blue, alpha. The x and y
coordinates specify the upper right corner. Color values (including alpha,
which is opacity) should range from 0 to 255. For line the arguments are
the remaining arguments are x-start, y-start, x-end, y-end, red, green,
blue, alpha."
  (swap! shapes conj shape)
  (.paint panel (.getGraphics panel)))

(defn run []
  
  (draw-shape 'rect 20 20 460 80 255 128 0 255)
  (draw-shape 'oval 120 120 300 100 100 128 0 255)
  (draw-shape 'line 0 0 500 500 255 0 0 255)
)
